<?php
require_once('plugins/recipes/model/ImageLink.php');
require_once('plugins/recipes/model/ImageLinkFactory.php');
require_once('framework/model/Services.php');

/**
 * Services for an image link
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Thomas Sterren
 * @package org.brim-project.plugins.recipes
 * @subpackage model
 * @filesource
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2006 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class ImageLinkServices extends Services
{
	/**
	 * Default constructor
	 */
	function ImageLinkServices()
	{
		parent::Services();
		$this->itemFactory = new ImageLinkFactory();

		include ('plugins/recipes/sql/imagelinkQueries.php');
		$this->queries = $queries;
	}

	/**
	 * Adds information for a image link
	 *
	 * @param integer userId the identifier for the user
	 * @param object item the image link to be added
	 *
     * @return integer last know id. This is the id of the recipe
	 * that was newly inserted and for which an id has automatically
	 * been assigned.
	 */
	function addItem($userId, $item)
	{
		$now = date("Y-m-d H:i:s");

		if (!isset($item->guid))
		{
			$item->guid = $this->itemFactory->createGUID();
		}

		// execute the query
		$query = sprintf($this->queries['addItem'],
			$now,
			$item->guid,
			$item->recipeId,
			addslashes(trim($item->position)),
			$item->arrangement,
			$item->mainImage,
			addslashes(trim($item->url)),
			addslashes(trim($item->class)),
			addslashes(trim($item->style)),
			($item->width != '') ? $item->width : 'NULL',
			($item->height != '') ? $item->height : 'NULL',
			$item->border,
			addslashes(trim($item->align)),
			($item->hSpace != '') ? $item->hSpace : 'NULL',
			($item->vSpace != '') ? $item->vSpace : 'NULL',
			addslashes(trim($item->alt)),
			addslashes(trim($item->title)),
			addslashes(trim($item->mediaRef)));

		$this->db->Execute($query) or
				die ("Could not add image link.".
				$this->db->ErrorMsg () . " " . $query . '<br />');

		// fetch the last known id
		// todo this is generic code and is candidate to move to the base Services class
		$query = $this->queries['lastItemInsertId'];
		$result = $this->db->Execute($query) or
			die ("Could not retrieve last insert id for image link. ".
			$this->db->ErrorMsg () . " " . $query . '<br />');

		// Rearrange arrangement entries of other items
		$this->rearrangeArrangementIds($item->recipeId,
									   $item->arrangement,
									   1,
									   $result->fields[0]);

        return $result->fields[0];
	}

	function addItems($recipeId, $items)
	{
		$itemIds = array();

		for ($i = 0; $i < count($items); $i++)
		{
			$itemIds[] = $this->addItem($recipeId, $items[$i], false);
		}
	}

	/**
	 * Modifies te recipe
	 *
 	 * @param integer userId the identifier for the user who modifies a recipe
	 * @param object item the modified recipe
	 */
	function modifyItem($userId, $item)
	{
		$now = date ("Y-m-d H:i:s");

		$query = sprintf ($this->queries['modifyItem'],
			$item->isDeleted,
			$now,							// when_modified
			$item->recipeId,
			$item->position,
			$item->arrangement,
			$item->mainImage,
			addslashes($item->url),
			addslashes($item->class),
			addslashes($item->style),
			$item->width,
			$item->height,
			$item->border,
			addslashes($item->align),
			$item->hSpace,
			$item->vSpace,
			addslashes($item->alt),
			addslashes($item->title),
			addslashes($item->mediaRef),
			$item->itemId);

		$this->db->Execute($query)
			or die ("Error modifying item: " .
	 		$this->db->ErrorMsg() . " " . $query . '<br />');
	}

	function modifyItems($recipeId, $items)
	{
		$oldIngredientCount = $this->getIngredientCount($recipeId);

		for ($i = 0; $i < count($items); $i++)
		{
			if ($items[$i]->itemId != 0)
			{
				$this->modifyItem(null, $items[$i]);
			}
			else
			{
				// We have a new ingredients. Add them
				$this->addItem($recipeId, $items[$i], true);
			}
		}

		if ($oldIngredientCount > count($items))
		{
			// Remove messy ingredients
			$this->removeMessyIngredients($recipeId, $oldIngredientCount - count($items));
		}
	}

	function deleteItem($userId, $itemId)
	{
		$item = $this->getItem($userId, $itemId);

		if (isset($item))
		{
			// now delete the specific item
			$query = sprintf($this->queries['deleteItem'], $itemId);

			$result = $this->db->Execute($query) or
				die("ImageLinkServices::deleteItem: " .
				$this->db->ErrorMsg() . " " . $query. '<br />');
		}
	}

	function deleteItems($userId, $recipeId)
	{
		$itemIds = $this->getIds($recipeId);

		for ($i = 0; $i < count($itemIds); $i++)
		{
			$this->deleteItem($userId, $itemIds[$i]);
		}
	}

	/**
	* Search for items for a user
	*
	* @param string userId the identifier for the user for which we
	*  	would like to search for items
	* @param string field the field-array on which we would like to search
	* @param string value the value for which we would like to search
	*
	* @return array all items that match the given search criteria
	*/
	function searchItems ($userId, $field, $value)
	{
		$query = sprintf($this->queries ['searchItems'],
						 $field,
						 $value,
						 $userId);

		$result = $this->db->Execute ($query) or die
				("Operations. Search " . $this->db->ErrorMsg () .
					" " . $query);

		$items = $this->itemFactory->resultsetToItems($result);
		return $items;
	}

	/**
	* Search an expression in multible fields.
	*
	* @param string userId the identifier for the user for which we
	*  	would like to search for items
	* @param string fields the field-array on which we would like to search
	* @param string value the value for which we would like to search
	*
	* @return array all items that match the given search criteria
	*/
	function searchExpressionInMultipleFieldsItems($userId, $fields, $value, $navigationMode)
	{
		$query = sprintf($this->queries ['searchExpressionInMultipleFieldsItems'],
						 $fields[0], $value,
						 $fields[1], $value,
						 $fields[2], $value,
						 $fields[3], $value,
						 $fields[4], $value,
						 $fields[5], $value,
						 $userId);

		$result = $this->db->Execute ($query) or die
				("Operations. Search " . $this->db->ErrorMsg () .
					" " . $query);

		$items = $this->itemFactory->resultsetToItems($result);
		return $items;
	}

	/**
	*
	*/
	function getImageLinks($recipeId)
	{
		return $this->getImages($recipeId);
	}

	function getImages($recipeId, $position='topRight')
	{
		$query = sprintf($this->queries['getImages'],
						 $recipeId,
						 $position);
		$result = $this->db->Execute($query) or
			die("ImageLinkServices::getImages: " . $this->db->ErrorMsg());

		return $this->itemFactory->resultsetToItems($result);
	}

	function getImageLinkCount($recipeId)
	{
		$query = sprintf($this->queries ['getImageLinkCount'],
						 $recipeId);

		$result = $this->db->Execute ($query) or die
				("Operations. getImageLinkCount " . $this->db->ErrorMsg () .
					" " . $query);

        return (isset($result->fields[0])) ? $result->fields[0] : 0;
	}

	/**
	* Deletes the last ingredients, which are too much
	*/
	function removeMessyImageLinks($recipeId, $messyIngredientCount)
	{

	}

	/*
	 * Rearranges arrangement entries of other items
	 */
	function rearrangeArrangementIds($recipeId, $arrangement, $offset = 1, $exludeItemId = NULL)
	{
		$query = sprintf($this->queries['getOrders'],
						 $recipeId,
						 $arrangement);

		$result = $this->db->Execute($query) or die
						("Operations. rearrangeArrangementIds " . $this->db->ErrorMsg () .
						" " . $query);

		if (!isset($exludeItemId))
		{
			$exludeItemId = 0;
		}

		while (!$result->EOF)
		{
			if ($exludeItemId != $result->fields['item_id'])
			{
				$this->modifyOrderId($result->fields['item_id'], $result->fields['arrangement'] + $offset);
			}

			$result->MoveNext();
		}
	}

	function modifyOrderId($itemId, $newOrderId)
	{
		if (isset($itemId) &&
			isset($newOrderId) &&
			$itemId > 0 &&
			$newOrderId > 0)
		{
			$query = sprintf($this->queries['modifyOrderId'],
							 $newOrderId,
							 $itemId);

			$this->db->Execute($query)
				or die ("Error modifying arrangement: " .
				$this->db->ErrorMsg() . " " . $query . '<br />');
		}
	}

	function getIds($recipeId, $order='ASC')
	{
		$itemIds = array();

		$query = sprintf($this->queries['getIds'],
						 $recipeId,
						 $order);

		$result = $this->db->Execute ($query) or die
				("Operations. getIds " . $this->db->ErrorMsg() .
			" " . $query);

		while (!$result->EOF)
		{
			$itemIds[] = $result->fields[0];
			$result->MoveNext();
		}

		return $itemIds;
	}

	function deleteAllItemsForUser($userId, $recipeIds)
	{
		for ($i = 0; $i < count($recipeIds); $i++)
		{
			$this->deleteItems($userId, $recipeIds[$i]);
		}
	}
}
?>
