<?php
require_once('plugins/recipes/model/Recipe.php');
require_once('plugins/recipes/model/RecipeFactory.php');
require_once('framework/model/Services.php');
require_once('plugins/recipes/model/LinkServices.php');
require_once('plugins/recipes/model/ImageLinkServices.php');

/**
 * Services for a Recipe
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Thomas Sterren
 * @package org.brim-project.plugins.recipes
 * @subpackage model
 * @filesource
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2006 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class RecipeServices extends Services
{
	var $linkServices;
	var $imageLinkServices;

	/**
	 * Default constructor
	 */
	function RecipeServices()
	{
		parent::Services();

		$this->itemFactory = null;
		$this->linkServices = new LinkServices();
		$this->imageLinkServices = new ImageLinkServices();

		include ('plugins/recipes/sql/recipeQueries.php');
		$this->queries = $queries;
	}

	function setFactory(&$theFactory)
	{
		$this->itemFactory = $theFactory;

		$this->linkServices->setFactory($this->itemFactory);
	}

	/**
	 * Gets a specific item for a user.
	 * Gives an empty item when the query in the database gives
	 * no result. Thereout the renderer gets always an item. For
	 * example when adding a new item
	 *
	 * @param string userId the identifier for the user
	 * @param string itemId the identifier for the item
	 * @return object the item for specified user with specified itemId
	 */
	function getItem ($userId, $itemId)
	{
		$query = sprintf ($this->queries['getItem'], $itemId);

		$result = $this->db->Execute($query) or
			die("GetItem: ". $this->db->ErrorMsg()." query: -". $query . "-");

		$items = $this->itemFactory->resultsetToItems ($result);
		if (count ($items) == 0)
		{
			return $this->itemFactory->getEmptyItem();
		}
		else
		{
			return $items[0];
		}
	}

	/**
	 * Adds information for a recipe
	 *
	 * @param integer userId the identifier for the user
	 * @param object item the recipe to be added
	 *
     * @return integer last know id. This is the id of the recipe
	 * that was newly inserted and for which an id has automatically
	 * been assigned.
	 */
	function addItem($userId, $item)
	{
		$now = date("Y-m-d H:i:s");

		if (!isset($item->guid))
		{
			$item->guid = $this->itemFactory->createGUID();
		}

		// execute the query
		$query = sprintf($this->queries['addItem'],
			$userId,
			$item->parentId,
			$item->isParent,
			addslashes(trim($item->name)),
			addslashes(trim($item->description)),
			addslashes(trim($item->visibility)),
			addslashes(trim($item->category)),
			$now,
			$item->guid,
			$item->vegetarian,
			addslashes(trim($item->servings)),
			addslashes(trim($item->complexity)),
			addslashes(trim($item->duration)),
			addslashes(trim($item->costs)),
			addslashes(trim($item->country)),
			addslashes(trim($item->author)),
			addslashes(trim($item->url)),
			addslashes(trim($item->nutrition)),
			addslashes(trim($item->copyright)),
			addslashes(trim($item->user_field1)),
			addslashes(trim($item->user_field2)),
			addslashes(trim($item->user_field3)),
			addslashes(trim($item->user_field4)),
			addslashes(trim($item->user_field5)));

		$this->db->Execute($query) or
				die ("Could not add recipe.".
				$this->db->ErrorMsg () . " " . $query . '<br />');

		// fetch the last known id
		// todo this is generic code and is candidate to move to the base Services class
		$query = $this->queries['lastItemInsertId'];
		$result = $this->db->Execute($query) or
			die ("Could not retrieve last insert id for recipe. ".
			$this->db->ErrorMsg () . " " . $query . '<br />');

		$item->itemId = $result->fields[0];

		$this->addIngredients($item);
		$this->addTextSections($item);

        return $result->fields[0];
	}

	/**
	 * Modifies te recipe
	 *
 	 * @param integer userId the identifier for the user who modifies a recipe
	 * @param object item the modified recipe
	 */
	function modifyItem($userId, $item)
	{
		//die ("RecipeService::modifyItem ".bprint_r($item));
		$now = date ("Y-m-d H:i:s");

		$query = sprintf ($this->queries['modifyItem'],
			$item->parentId,
			addslashes($item->name),
			addslashes($item->description),
			$item->visibility,
			$item->isDeleted,
			$now,							// when_modified
			$item->vegetarian,
			addslashes($item->servings),
			addslashes($item->complexity),
			addslashes($item->duration),
			addslashes($item->costs),
			addslashes($item->country),
			addslashes($item->author),
			addslashes($item->url),
			addslashes($item->nutrition),
			addslashes($item->copyright),
			addslashes($item->user_field1),
			addslashes($item->user_field2),
			addslashes($item->user_field3),
			addslashes($item->user_field4),
			addslashes($item->user_field5),
			$item->itemId);

		$this->modifyIngredients($item);
		$this->modifyTextSections($item);

		$this->db->Execute($query)
			or die ("Error modifying item: " .
	 		$this->db->ErrorMsg() . " " . $query . '<br />');
	}

	function deleteItem($userId, $itemId)
	{
		$item = $this->getItem($userId, $itemId);

		if (isset($item))
		{
			// If we are a parent, delete the children as well....
			if ($item->isParent==1)
			{
				$children = $this->getChildren ($userId, $itemId);
				for ($i=0; $i<count($children); $i++)
				{
					$this->deleteItem ($userId, $children[$i]->itemId);
				}
			}
			else
			{
				// if the recipe has ingredients or text sections attached,
				// delete them as well
				$this->linkServices->deleteItems($userId, $itemId);
			}

			// now delete the specific item
			$query = sprintf($this->queries['deleteItem'], $itemId);

			$result = $this->db->Execute($query) or
				die("RecipeServices::DeleteItem: " .
				$this->db->ErrorMsg() . " " . $query. '<br />');
		}
	}

	/**
	 * Delete ALL items for the specified user
	 * @param string userId the name for the user of which all
	 * 	items needs to be deleted
	 */
	function deleteAllItemsForUser($userId)
	{
		// sterry FIXME: Needed when deleting a user
		if (!isset($this->itemFactory))
		{
			$this->setFactory(new RecipeFactory());
			$this->itemFactory->setServices($this);
		}
		$recipeIds = $this->getAllItemIdsForUser($userId);

		// Delete all ingredients and text sections for the user
		$this->linkServices->deleteAllItemsForUser($userId, $recipeIds);
		// Delete all image links
		$this->imageLinkServices->deleteAllItemsForUser($userId, $recipeIds);

		// Delete the recipes too
		$query = sprintf($this->queries['deleteAllForUser'], $userId);
		$this->db->Execute ($query);
	}

	/**
	* Search for items for a user
	*
	* @param string userId the identifier for the user for which we
	*  	would like to search for items
	* @param string field the field-array on which we would like to search
	* @param string value the value for which we would like to search
	*
	* @return array all items that match the given search criteria
	*/
	function searchItems($userId, $field, $value)
	{
		return $this->searchExpressionInMultipleFieldsItems($userId,
															'fulltext',
															$value,
															$_SESSION['navigationMode']);
	}

	/**
	* Search an expression in multible fields.
	*
	* @param string userId the identifier for the user for which we
	*  	would like to search for items
	* @param string fields the field-array on which we would like to search
	* @param string value the value for which we would like to search
	*
	* @return array all items that match the given search criteria
	*/
	function searchExpressionInMultipleFieldsItems($userId, $searchKey, $value, $navigationMode)
	{
		$result = null;
		$items = null;

		// sterry FIXME: Needed during overall search
		if (!isset($this->itemFactory))
		{
			$this->setFactory(new RecipeFactory());
			$this->itemFactory->setServices($this);
		}

		switch($searchKey)
		{
		default:
		case 'title':
			$result = $this->searchExpressionInMultipleFieldsResult($userId,
																	$value,
																	$navigationMode);
			$items = $this->itemFactory->resultsetToItems($result);
			break;
		case 'ingredients':
		case 'textsections':
			$result = $this->linkServices->searchExpressionInMultipleFieldsResult($userId,
																				  $searchKey,
																				  $value,
																				  $navigationMode);
			$items = $this->itemFactory->resultsetToItems($result);
			break;
		case 'fulltext':
			$result = $this->searchExpressionInMultipleFieldsResult($userId,
																	$value,
																	$navigationMode);
			$items = $this->itemFactory->resultsetToItems($result);

			$result = $this->linkServices->searchExpressionInMultipleFieldsResult($userId,
																				  'ingredients',
																				  $value,
																				  $navigationMode);
			$items = array_merge($items, $this->itemFactory->resultsetToItems($result));

			$result = $this->linkServices->searchExpressionInMultipleFieldsResult($userId,
																				  'textsections',
																				  $value,
																				  $navigationMode);
			$items = array_merge($items, $this->itemFactory->resultsetToItems($result));

			$this->removeDuplicateResults($items);
			break;
		}

		return $items;
	}

	function searchExpressionInMultipleFieldsResult($userId, $value, $navigationMode)
	{
		$fields = array();
		$whereExpression = '';
		$basicQuery = '';

		$fields[0] = 'name';
		$fields[1] = 'description';

		// Construct the where expression
		for ($i = 0; $i < count($fields); $i++)
		{
			$whereExpression .= $fields[$i] ." LIKE '%".$value."%'";
			$whereExpression .= ($i < (count($fields) - 1)) ? ' or ' : '';
		}

		if ($navigationMode != 'public')
		{
			$basicQuery = 'searchExpressionInMultipleFieldsItems';
		}
		else
		{
			$basicQuery = 'searchExpressionInMultipleFieldsItemsOrPublic';
		}

		$query = sprintf($this->queries [$basicQuery],
						 $whereExpression,
						 $userId);

		$result = $this->db->Execute ($query) or die
				("Operations. Search multiple fields" . $this->db->ErrorMsg () .
			" " . $query);

		return $result;
	}

	/**
	*
	*/
	function getIngredients($recipeId)
	{
		return $this->linkServices->getIngredients($recipeId);
	}

	/**
	*
	*/
	function getTextSections($recipeId)
	{
		return $this->linkServices->getTextSections($recipeId);
	}

	/**
	*
	*/
	function getLinks($recipeId)
	{
		return $this->linkServices->getLinks($recipeId);
	}

	/**
	*
	*/
	function getImageLinks($recipeId)
	{
		return $this->imageLinkServices->getImageLinks($recipeId);
	}


	function addIngredients($item)
	{
		$this->linkServices->addIngredients($item);
	}

	function addTextSections($item)
	{
		$this->linkServices->addTextSections($item);
	}

	function addLinks($item)
	{
		$this->linkServices->addItems($item);
	}

	function modifyIngredients($item)
	{
		$this->linkServices->modifyIngredients($item);
	}

	function modifyTextSections($item)
	{
		$this->linkServices->modifyTextSections($item);
	}

	function addImage($imageLink)
	{
		$this->imageLinkServices->addItem(null, $imageLink);
	}

	function getImageLinkItem($recipeId, $imageLinkId=0)
	{
		return $this->imageLinkServices->getItem($recipeId, $imageLinkId);
	}

	function modifyImage($imageLink)
	{
		$this->imageLinkServices->modifyItem(null, $imageLink);
	}

	function deleteImage($imageLinkItemId)
	{
		$this->imageLinkServices->deleteItem(null, $imageLinkItemId);
	}

	function removeDuplicateResults(&$items)
	{
		$items = array_unique_itemId($items);
	}

	function getAllItemIdsForUser($userId)
	{
		$itemIds = array();

		$query = sprintf($this->queries ['itemIdsForUser'],
						 $userId);

		$result = $this->db->Execute ($query) or die
				("Operations. getAllItemIdsForUser " . $this->db->ErrorMsg () .
					" " . $query);

		while (!$result->EOF)
		{
			$itemIds[] = $result->fields[0];

			$result->MoveNext();
		}

        return $itemIds;
	}
}
?>
