<?php

$log = fopen ('debug.log', 'w');

/**
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - April 2006
 * @package org.brim-project.plugins.weather
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2006 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */

/**
 * @author denis noessler
 * @link http://www.php.net/manual/en/function.array-diff-key.php#47630
 */
if (!function_exists ('array_diff_key')) 
{
	/**
	 * Computes the difference of arrays using keys for comparison
	 *
	 * @param array $valuesBase Base elements for comparison, associative
	 * @param array $valuesComp[,..] Comparison elements, associative
	 *
	 * @return array Elements, not existing in comparison element, associative
	 */
	function array_diff_key ()
	{
		$argCount = func_num_args ();
		$argValues = func_get_args ();
		$valuesDiff = array ();
		if ($argCount < 2)
		{
			return false;
		}
		foreach ($argValues as $argParam)
		{
			if (!is_array ($argParam))
			{
				return false;
			}
		}
		foreach ($argValues [0] as $valueKey => $valueData)
		{
			for ($i = 1; $i<$argCount; $i++)
			{
				if (isset ($argValues [$i][$valueKey]))
				{
					continue 2;
				}
			}
			$valuesDiff [$valueKey] = $valueData;
		}
		return $valuesDiff;
	}
}

/**
 * Log the provided string
 *
 * @param string $string
 */
function logstr ($string)
{
	global $log;
	fputs ($log, $string.'
');
}


/**
 * Write country information to a file. Errors will be written to
 * a seperate file as well as possible diffs with existing files
 * 
 * If the input is 'be' (countryCode for Belgium), there might be
 * three files at the end:
 * <ul>
 * <li>icao_be.php, containing the station information</li>
 * <li>diff_be.log, containing the diff between current information and previous</li>
 * <li>errors_be.log containing station names that were unreachable to fetch metar info</li>
 * </ul>
 */
function createInformationFiles ($countryCode, $input)
{
//
// The header for the result file
//
$header = '
/**
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * The information found in the file is retrieved from the
 * following location:
 * {@link http://www.rap.ucar.edu/weather/surface/stations.txt http://www.rap.ucar.edu/weather/surface/stations.txt}
 * The country codes are based on the ISO 3166-1 standard
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - April 2006
 * @package org.brim-project.plugins.weather
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2006 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
';
	
	//
	// We have a list of stations, grouped by country code.
	// These stations have names that come from the internet
	// site mentioned in the header.
	//
	// Now.. this stationName is in CAPS only.
	// Check existing station names and replace
	// if we have a better version
	//
	$fileName = '../util/include/icao_'.$countryCode.'.php';
	//
	// Reset the array
	//
	$icao = array ();
	if (file_exists ($fileName))
	{
		logstr ('Found an exitsing country file: '.$countryCode);
		include $fileName;
	}	
	else 
	{
		logstr ('File '.$fileName.' does not exist');;
	}
	//
	// Check whether we already have this name
	// And replace the found by the existing 
	//
	foreach ($input as $icaoCode => $stationName)
	{
		if (isset ($icao [$countryCode][$icaoCode]))
		{
			$stationName = $icao [$countryCode][$icaoCode];
			logstr ('StationName '.$stationName.' already in files, taking recorded version');
			// 
			// Replace the existing information
			//
			$input [$icaoCode] = $stationName;
		}
		else 
		{
			logstr ('StationName '.$stationName.' is new');
			//
			// Just add three stars for easy tracking
			//
			//$stationName = $stationName.'***';
		}
	}
	//
	// A collection of possible errors
	//
	$errors = array ();
	// 
	// Reset the array
	//
	unset ($icao);
	//
	// Loop over the input
	//
	foreach ($input as $icaoCode => $stationName)
	{
		//
		// Now try to download the metar information
		// from the web. Crosscheck to see whether 
		// the station is valid
		//
		// If the station is not valid, remove it from
		// the input array and place it in the error
		// array
		//
		if (!(@file ('http://weather.noaa.gov/pub/data/observations/metar/stations/'.$icaoCode.'.TXT')))
		{
			logstr ('looking for existence of '.$icaoCode.' at noaa. NOT FOUND');
			unset ($input [$icaoCode]);
			$errors [$countryCode][$icaoCode] = $stationName;
		}
		else 
		{
			logstr ('looking for existence of '.$icaoCode.' at noaa. Found');
		}
	}	
	if (count ($errors) > 0)
	{
		//
		// Write the error log
		//
		$handle = fopen ('errors_'.$countryCode.'.log', 'w');
		fputs ($handle, var_export ($errors, true));
		fclose ($handle);
	}
	//
	// Now we have a valid input, we can write it to file/
	// We do a cross-check with existing information
	// however and log it as well.
	//
	foreach ($input as $icaoCode => $stationName)
	{
		$fileName = '../util/include/icao_'.$countryCode.'.php';
		if (file_exists ($fileName))
		{
			include $fileName;
			//
			// Now check for the differences
			//
			$diff = array_diff_key ($icao [$icaoCode], $input);
			if (count ($diff) > 0)
			{
				$handle = fopen ('diff_'.$countryCode.'.log', 'w');
				fputs ($handle, 'Difference between '.
					var_export ($icao [$countryCode], true).' and '.
						var_export ($input, true));
				fclose ($handle);
			}
		}
		$resultString = '';
		foreach ($input as $icaoCode => $stationName)
		{
			$resultString .= "\$icao ['".$countryCode."']['".$icaoCode."'] = '".
				preg_replace ( "/'/", "\\'",$stationName)."';
";
		}
		//
		// Ok, write the actual info
		//
		$handle = fopen ('icao_'.$countryCode.'.php', 'w');
		$result = '<?php
'.$header.'
if (!isset ($icao))
{
	$icao = array ();
}
'.$resultString.'
?>';
		fputs ($handle, $result);
		fclose ($handle);
	}
}

logstr ('Opening internet-connection to stations.txt file');
//
// The information from the web
//
$lines = @file ('http://www.rap.ucar.edu/weather/surface/stations.txt');
//$lines = @file ('stations.txt');
logstr ('File downloaded');
//
// The array that will contain ALL information. The array has the following structure:
//
// [countrycode] [station name]
//
$countries = array ();

//
// Loop over all lines
//
foreach ($lines as $line)
{
	$hasMetar = substr ($line, 62, 1) == 'X';
	//
	// All countries are sorted. Check with previous country
	// code and if there is a difference (and therefor a new
	// country), some things can be reset
	//
	if ($hasMetar)
	{
		logstr ('Found the following line (containing METAR info): '.$line);
		// 
		// Fetch the countryCode, the stationName and the 
		// icao code from the file.
		//
		$countryCode = strtolower(substr ($line, 81, 2));
		$stationName = trim (substr ($line, 3, 17));
		$icaoCode = trim (substr ($line, 20, 4));
		//
		// Ok, information is assembled. Add it to the array
		//
		$countries [$countryCode][$icaoCode] = $stationName;
	}
}
//
// Since the list is not sorted by countrycode, we need to 
// fetch all information first before we can start any processing
//
foreach ($countries as $countryCode => $icaoCodeAndStationName)
{
	createInformationFiles ($countryCode, $countries[$countryCode]);
}
//
// Now, don't forget the last countries
//
//logstr ('Closing with last country '.$countryCode);
//createInformationFiles ($countryCodeCountry, $countries [$countryCode]);

fclose ($log);
?>
