<?php

require_once ('plugins/weather/util/ICAOUtils.php');
/**
 * A Metar/Taf helper class.
 * 
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - April 2006
 * @package org.brim-project.plugins.notes
 * @subpackage sql
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2006 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */

class MetarTafHelper
{
	var $icaoUtils;
	
	function MetarTafHelper ()
	{
		$this->icaoUtils = new ICAOUtils ();	
	}
	
	
	/**
	 * Transform weather information to icons.
	 * 
	 * The name of the icons will always start with
	 * four digits representing the sevrity and
	 * an optional appendix (preprended by an underscore) 
	 * indicating fog or thunder
	 * 
	 * The first digit represents cloud coverage
	 * where
	 * 0 is no coverage
	 * 1 is few
	 * 2 is scattered
	 * 3 is broken
	 * 4 is overcast
	 *
	 * The second digit indicates rain where
	 * 0 is no rain
	 * 1 is light rain
	 * 2 is moderate rain
	 * 3 is heavy rain
	 * 
	 * The third digit indicates hail where
	 * 0 is no hail
	 * 1 is light hail
	 * 2 is moderate hail
	 * 3 is heavy hail
	 * 
	 * The fourth digit indicates snow where
	 * 0 is no snow
	 * 1 is light snow
	 * 2 is moderate snow
	 * 3 is heavy snow
	 * 
	 * Optionally there are the following appendixes:
	 * _fog
	 * _thunder
	 * 
	 * @param array $info
	 * @return string 
	 */
	function transformWeatherInformationToWeatherIcon ($info)
	{
		if (!isset ($info['temperature']))
		{
			return 'day_unknown.png';
		}
		$coverage = $this->getCloudCoverageIndex ($info);
		$rain = $this->getRainIndex ($info);
		$snow = $this->getSnowIndex ($info);
		$hail = $this->getHailIndex ($info);
		$appendix = $this->getAppendix ($info);
		return 'day_'.$coverage.$rain.$snow.$hail.$appendix.'.png';
	}
	
	function transformWeatherInformationToTemperatureIcon ($info)
	{
		$temp = $info ['temperature']['temperature'];
		if (isset ($temp))
		{
			if ($temp < 0)
			{
				$img = 'freeze';
			}
			else if ($temp < 10)
			{
				$img = 'cold';
			}
			else if ($temp < 20)
			{
				$img = 'low';
			}
			else if ($temp < 30)
			{
				$img = 'warm';
			}
			else if ($temp >= 30) 
			{
				$img = 'hot';
			}
		}
		else 
		{
			$img = 'unknown';
		}
		return 'temperature_'.$img.'.png';
	}

	function transformWeatherInformationToWindIcon ($info)
	{
		$index = $info ['wind']['directionCompass'];
		if (isset ($index))
		{
			return 'wind_'.strtolower($index).'.png';
		}
		return 'wind_unknown.png';
	}
	/**
	 * Enter description here...
	 *
	 * @param unknown_type $info
	 * @return unknown
	 * @private
	 */
	function getCloudCoverageIndex ($info)
	{
		//
		// No cloud coverage
		//
		if (!isset ($info['cloudCoverage']))
		{
			return 0;
		}
		//
		// Lets take the first one, if more
		//
		$input = $info ['cloudCoverage'][0]['coverageElement'];
		if ($input == 'SKC' || $input == 'CLR' ||
			$input == 'CAVOK' || $input == 'NSC' || $input == 'NCD')
		{
			return 0;
		}
		else if (strstr ($input, 'VV')) 
		{
			return 3;
		}
		else if (strstr ($input, 'FEW'))
		{
			if (isset ($info ['cloudCoverage'][1]) &&
				($info ['cloudCoverage'][1]['coverageElement']) == 'SCT')
			{
				//
				// BXL Gives 'FEW SCT BKN' a lot which translates 
				// into 'mostly clouds'
				//
				return 2;
			}
			return 1;
		}
		else if (strstr ($input, 'SCT'))
		{
			return 2;
		}
		else if (strstr ($input, 'BKN'))
		{
			return 3;
		}
		else if (strstr ($input, 'OVC'))
		{
			return 4;
		}
		// default (unknown?)
		return 0;
	}
	
	/**
	 * Enter description here...
	 *
	 * @param unknown_type $info
	 * @return unknown
	 * @private
	 */
	function getRainIndex ($info)
	{
		// 
		// Multiple weather elements can be available. Take the first one
		//
		if (isset ($info ['weatherConditions'][0]['precipation']) &&
			$info ['weatherConditions'][0]['precipation'] == 'rain')
		{
			if (isset ($info ['weatherConditions'][0]['intensity']))
			{
				switch ($info['weatherConditions'][0]['intensity'])
				{
					case "light":
						return 1;
					case "heavy":
						return 3;
				}
				// no intensity -> moderate
				return 2;
			}
		}
		else if (isset ($info ['weatherConditions'][0]['precipation']) &&
			$info ['weatherConditions'][0]['precipation'] == 'drizzle')
		{
			// drizzle is light rain
			return 1;
		}
		// no rain (or unknown?)
		return 0; 
	}

	/**
	 * Enter description here...
	 *
	 * @param unknown_type $info
	 * @return unknown
	 * @private
	 */
	function getSnowIndex ($info)
	{
		if (isset ($info ['weatherConditions']['precipation']) &&
			($info ['weatherConditions']['precipation'] == 'snow'))
		{
			if (isset ($info ['weatherConditions']['intensity']))
			{
				switch ($info ['weatherConditions']['intensity'])
				{
					case "light":
						return 1;
					case "heavy":
						return 3;
				}
				// no intensity -> moderate
				return 2;
			}
		}
		else if (isset ($info ['weatherConditions']['precipation']) &&
			($info ['weatherConditions']['precipation'] == 'snowgrains'))
		{
			// lets say that snowgrains equals light snow
			return 1;			
		}
		// no snow (or unknown?)
		return 0;
	}
	
	/**
	 * Enter description here...
	 *
	 * @param unknown_type $info
	 * @return unknown
	 * @private
	 */
	function getHailIndex ($info)
	{
		if (isset ($info ['weatherConditions']['precipation']) &&
			($info ['weatherConditions']['precipation'] == 'hail'))
		{
			if (isset ($info ['weatherConditions']['intensity']))
			{
				switch ($info ['weatherConditions']['intensity'])
				{
					case "light":
						return 1;
					case "heavy":
						return 3;
				}
				// no intensity -> moderate
				return 2;
			}
		}
		else if (isset ($info ['weatherConditions']['precipation']) &&
			($info ['weatherConditions']['precipation'] == 'smallhail'))
		{
			// light
			return 1;			
		}
		else if (isset ($info ['precipation']) &&
			($info ['precipation'] == 'ice_pellets'))
		{
			// heavy
			return 3;			
		}
		// no hail (or unknown?)
		return 0;
	}
	
	/**
	 * Enter description here...
	 *
	 * @param unknown_type $info
	 * @return unknown
	 * @private
	 */
	function getAppendix ($info)
	{
		if (isset ($info ['description']) && $info ['description'] == 'thunderstorm')
		{
			return '_thunder';
		}
		else if (isset ($info ['obscuration']) && $info ['obscuration'] == 'fog')
		{
			return '_fog';
		}
		// ignore the rest for the moment
		else return '';
	}
	
	function getInfoAsHtml ($info, $dictionary)
	{
		$station = str_replace(', ','<br />',
			$this->icaoUtils->getStationForICAOCode ($info['station']));
		$country = $this->icaoUtils->getCountryForICAOCode ($info['station']);
		$result  = '
		<table border="1" cellspacing="0" cellpadding="0">
			<tr>
				<td>
					<b>'.$dictionary['generic'].'</b>
				</td>
				<td>&nbsp;</td>
			</tr>	
			<tr>
				<td>
					'.$dictionary['station'].'
				</td>
				<td>
					'.$station.'
				</td>
			</tr>	
			<tr>
				<td>
					'.$dictionary['country'].'
				</td>
				<td>
					'.$country.'
				</td>
			</tr>	
			<tr>
				<td>
					<b>'.$dictionary['cloudReport'].'</b>
				</td>
				<td>
					&nbsp;
				</td>
			</tr>	
		</table>';
		return '';
	}
}
?>
