<?php

/**
 * A Metar/Taf parser. Is capable of fetching METAR/TAF information from the
 * web and parsing it, returing an array of weather information
 * 
 * Information on Metar/Taf can be found at the folowing locations:
 * 
 * http://www.nws.noaa.gov/oso/oso1/oso12/fmh1/fmh1ch12.htm
 * http://www.wunderground.com/metarFAQ.asp
 * http://www.astro.keele.ac.uk/oldusers/rno/Aviation/metar_codes.html
 * http://www.vnv-dalpa.com/weather/decoding.htm
 * http://www.sailned.com/sailned/metar.htm
 * http://aviation.weathersa.co.za/codesexpl.htm
 * http://www.met.tamu.edu/class/METAR/metar.html#INDEX
 * http://aviation.weathersa.co.za/Aviationcodes/Aviation.htm
 * http://www.airport-technology.com/icao-codes/
 * 
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - April 2006
 * @package org.brim-project.plugins.notes
 * @subpackage sql
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2006 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */

class MetarTafParser
{
	/**
	 * Default constructor
	 *
	 * @return MetarTafUtils
	 */
	function MetarTafParser ()
	{
	}
	
	/**
	 * Retrieve the METAR/TAF information from the web, based on the stations
	 * ICAO code.
	 *
	 * This function returns an array with two elements:
	 * <ul>
	 * <li><b>observed</b>, datetime of last observation</li>
	 * <li><b>metar/taf</b> (one of them, based on input), containing the 
	 * metar/taf string</li>
	 * </ul>
	 * 
	 * @param string $stationICAOCode
	 * @param string $which the variable must be either TAF or METAR
	 * @return array
	 * @private
	 */
	function getMetarTafFromWeb ($stationICAOCode, $which)
	{
		$result = null;
		$which = strtolower ($which);
		if ($which == 'metar')
		{
			$url = 'http://weather.noaa.gov/pub/data/observations/metar/stations/';
		}
		else if ($which == 'taf')
		{
			$url = 'http://weather.noaa.gov/pub/data/forecasts/taf/stations/';
		}
		//die (print_r ($url.$stationICAOCode.'.TXT'));
		$retrievedResult = @file ($url.$stationICAOCode.'.TXT');
		if (isset ($retrievedResult) && count ($retrievedResult) > 0) 
		{
			$result = array ();
			$explodedItemArray = explode (' ', $retrievedResult [0]);
			$result [$which.'Observed'] = trim ($retrievedResult [0]);
			if (!isset ($result [$which.'Observed']) || $result [$which.'Observed'] == '')
			{
				return null;
			}
			for ($i=1; $i<count ($retrievedResult); $i++) 
			{
				// so either 'taf' or 'metar' is used as array index
		   		$result [$which] .= ' '.trim ($retrievedResult [$i]);
			}
			$result [$which] = trim ($result [$which]);
		}
		if ($which == 'taf')
		{
			//die (print_r ($result));
		}
		return $result;
	}

	function parse ($input, $which)
	{
		if (strtolower ($which) == 'metar')
		{
			return $this->parseMetar ($input);
		}
		else if (strtolower ($which) == 'taf')
		{
			return $this->parseTaf ($input);
		}
		else
		{
			return null;
		}
	}

	function parseTaf ($input)
	{
		$result = array ();
		$inputArray = explode (' ', $input);
		array_walk ($inputArray, 'trim');
		$pivot = 0;
		$result ['taf'] = $input;
		//
		// station is always present
		//
		$result ['station'] = $inputArray [$pivot++];
		if (!isset ($result['station']))
		{
			return $result;
		}
		$modifier = $this->getModifier ($inputArray [$pivot]);
		while (isset ($modifier))
		{
			$pivot++;
			$result ['modifier'][] = $modifier;
			$modifier = $this->getModifier ($inputArray [$pivot]);
		}
		//
		// Get the observed. Optional??
		$observed = $this->getMetarTafTime ($inputArray [$pivot]);	
		if (isset ($observed))
		{
			$pivot++;
			$result ['tafObservedDDHHMM'] = $observed;
		}
		//
		// Get the validity
		//
		$result ['validity'] = $this->getValidity ($inputArray [$pivot++]);
		//
		// Look for wind information
		//
		$result ['wind'] = $this->getWind ($inputArray [$pivot++]);
		//
		// Get the visibility
		//
		// If the visibility is in miles AND fractional, the fraction is 
		// seperated by a space. This screws up our initial split on spaces
		// so we do a check on whether we have received a single integer 
		// (in the calling function) and if this is the case, the function 
		// is called a second time to determine the fraction)
		//
		$visibility = $this->getVisibility ($inputArray [$pivot++]);
		if (!isset ($visibility ['units']) && strlen ($visibility['visibility'] == 1))
		{
			$visibility ['units'] = 'miles';
			$visibility ['fraction'] = $this->getVisibility ($inputArray [$pivot++]);
		}
		$result ['visibility'] = $visibility;
		//
		// Get the weather conditions.
		// The conditions are deliberately not translated!
		//
		$weatherConditions = 
			$this->getWeatherConditions ($inputArray [$pivot]);
		while (isset ($weatherConditions))
		{
			$pivot++;
			$result ['weatherConditions'][] = $weatherConditions;
			$weatherConditions = 
				$this->getWeatherConditions ($inputArray [$pivot]);
		}
		//
		// Get the cloud coverage information
		//
		$cloudCoverage = $this->getCloudCoverage ($inputArray [$pivot]);
		while (isset ($cloudCoverage))
		{
			$pivot++;
			$result['cloudCoverage'][] = $cloudCoverage;

			$cloudCoverage = $this->getCloudCoverage 
				($inputArray [$pivot]);
		}
		//
		// Get the wind shear
		//
		$windShear = $this->getWindShear ($inputArray [$pivot]);
		if (isset ($windShear))
		{
			$pivot++;
			$result ['windShear'] = $windShear;
		}
		//
		// altimeter setting/vertical visibility?
		//
		
		
		//
		// Icing group
		//
		$icingGroup = $this->getIcingGroup [$pivot];
		if (isset ($icingGroup))
		{
			$pivot++;
			$result ['icingGroup'] = $icingGroup;
		}
		//
		// Turbulence
		//
		$turbulence = $this->getTurbulende [$pivot];
		if (isset ($turbulence))
		{
			$pivot++;
			$result ['turbulence'] = $turbulence;
		}
		
		//
		// Probability
		//
		$probability = $this->getProbability ($inputArray [$pivot]);
		if (isset ($probability))
		{
			$pivot++;
			$result ['probability'] = $probability;
		}
		return $result;
	}

	function parseMetar ($input)
	{
		$result = array ();
		$inputArray = explode (' ', $input);
		array_walk ($inputArray, 'trim');
		$pivot = 0;
		$result ['metar'] = $input;
		//
		// station is always present
		//
		$result ['station'] = $inputArray [$pivot++];
		if (!isset ($result['station']))
		{
			return $result;
		}
		//
		// Look for observation time
		//
		$result ['metarObservedDDHHMM'] = $this->getMetarTafTime ($inputArray [$pivot++]);
		//
		// Look for OPTIONAL station type
		//
		$stationType = $this->getStationType ($inputArray [$pivot]);
		if (isset ($stationType))
		{
			$pivot++;
			$result ['stationType'] = $stationType;
		}
		//
		// Look for wind information
		//
		$result ['wind'] = $this->getWind ($inputArray [$pivot++]);
		//
		// Look for OPTIONAL variable wind information
		//
		$variableWind = $this->getVariableWind ($inputArray [$pivot]);
		if (isset ($variableWind))
		{
			$pivot++;
			$result ['variableWind'] = $variableWind;
		}
		//
		// Get the visibility
		//
		// If the visibility is in miles AND fractional, the fraction is 
		// seperated by a space. This screws up our initial split on spaces
		// so we do a check on whether we have received a single integer 
		// (in the calling function) and if this is the case, the function 
		// is called a second time to determine the fraction)
		//
		$visibility = $this->getVisibility ($inputArray [$pivot++]);
		if (!isset ($visibility ['units']) && strlen ($visibility['visibility'] == 1))
		{
			$visibility ['units'] = 'miles';
			$visibility ['fraction'] = $this->getVisibility ($inputArray [$pivot++]);
		}
		$result ['visibility'] = $visibility;
		//
		// Runway information. Multipe runways may be included, so keep checking 
		// until no runway information is found.
		//
		$runwayInfo = $this->getRunwayInfo ($inputArray [$pivot]);
		while (isset ($runwayInfo))
		{
			$pivot++;
			$result ['runwayInfo'][] = $runwayInfo;
			$runwayInfo = $this->getRunwayInfo ($inputArray [$pivot]);
		}
		//
		// Get the weather conditions.
		// The conditions are deliberately not translated!
		//
		$weatherConditions = 
			$this->getWeatherConditions ($inputArray [$pivot]);
		while (isset ($weatherConditions))
		{
			$pivot++;
			$result ['weatherConditions'][] = $weatherConditions;
			$weatherConditions = 
				$this->getWeatherConditions ($inputArray [$pivot]);
		}
		//
		// Get the cloud coverage information
		//
		$cloudCoverage = $this->getCloudCoverage ($inputArray [$pivot]);
		while (isset ($cloudCoverage))
		{
			$pivot++;
			$result['cloudCoverage'][] = $cloudCoverage;

			$cloudCoverage = $this->getCloudCoverage 
				($inputArray [$pivot]);
		}
		//
		// Get temperature and dew point information
		//
		$result ['temperature'] = $this->getTemperature 
			($inputArray [$pivot++]);
		//
		// Get pressure information
		//
		$pressure = $this->getPressure ($inputArray [$pivot]);
		if (isset ($pressure))
		{
			$pivot++;
			$result ['pressure'] = $pressure;
		}
//
// TODO REMARKS, TRENDS ETC
//
		return $result;
	}
	
	/**
	 * Returns the string that represent the time from the given string 
	 * (which is a part of a METAR/TAF string) or <code>null</code>
	 * if not found.
	 * The format of the return string is the following:
	 * ddhhmm where dd = day, hh = hours, mm = minutes. UTC time.
	 *
	 * @param string $input
	 * @return array
	 * @private
	 */
	function getMetarTafTime ($input)
	{
		$result = null;
		if (substr ($input, -1) == 'Z')
		{
			$result = array ();
			$result ['day'] = substr ($input, 0, 2);
			$result ['hours'] = substr ($input, 2, 2);
			$result ['minutes'] = substr ($input, 4, 2);
		}
		return $result;
	}
	
	/**
	 * Returns the stationtype for the given string.
	 * Either 'AUTO', 'COR' or <code>null</code> if not found
	 * 
	 * @param string $input
	 * @return string
	 * @private
	 */
	function getStationType ($input)
	{
		if ($input == 'AUTO' || $input == 'COR')
		{
			return $input;
		}
		return null;
	}
	
	/**
	 * Retuns wind information, based on the input.
	 * Result is an array containing the following elements:
	 * <ul>
	 * <li><b>direction</b>, the direction in degrees. 090 indicates E (East),
	 * 250 indicates WSW (WestSouthWest), 000 indicates no wind (logically, the 
	 * speed parameter will also be zero), VRB indicates variable direction</li>
	 * <li><b>speed</b>, the average windspeed over a period of 10 minutes prior 
	 * to the observation time. The unit of the speed are indicated in a dedicated
	 * array variable (unit)</li>
	 * <li><b>gust (optional!)</b>. The gust over the last 10 minutes, usually
	 * only indicated if there is a difference of 10 knots or more between the averag
	 * and the gust</li>
	 * <li><b>units</b>, the unit for the speed. Either KT (knots), MPS (meters per 
	 * second) or KMH (kilometers per hour)</li>
	 * </ul>
	 * 
	 * The function returns <code>null</code> if no weather information is found
	 * in the input string.
	 * 
	 * @param string
	 * @return array
	 * @private
	 */
	function getWind ($input)
	{
		$result = null;
		if (strstr ($input, 'KT') || strstr ($input, 'MPS') || strstr ($input, 'KMH'))
		{
			$result = array ();
			$direction = substr ($input, 0, 3);
			$result ['direction'] = $direction;
			$result ['directionCompass'] = $this->getCompassDirection ($direction);
			$result ['speed'] = substr ($input, 3, 2);
			$remainder = substr ($input, 5);
			if (strstr ($remainder, 'G'))
			{
				$result ['gust'] = substr ($input, 6, 2);
				$result ['units'] = substr ($input, 8);
			}
			else 
			{
				$result ['units'] = substr ($input, 5);
			}
		}
		return $result;
	}
	
	/**
	 * Returns variable wind information (this information is optional in METAR
	 * information) or <code>null</code> if not found. The input information
	 * must be in the form of fffVttt where 'fff' stands for the 'from' degrees and
	 * 'ttt' the 'to' degrees.
	 * 
	 * Variable wind is wind for which the direction varies 60 degrees or more 
	 * and wind is greater than 6 knots.
	 *
	 * @param string $input
	 * @return 
	 * @private
	 */
	function getVariableWind ($input)
	{
		$result = null;
		//
		// Information in the form of fffVttt
		//
		if (ereg ('([0-9]{3})V([0-9]{3})', $input))
		{
			$result = array ();
			$fromTo = explode ('V', $input);
			$result ['from'] = $fromTo [0];
			$result ['fromCompass'] = $this->getCompassDirection ($fromTo [0]);
			$result ['to'] = $fromTo [1];
			$result ['toCompass'] = $this->getCompassDirection ($fromTo [1]);
		}
		return $result;
	}
	
	/**
	 * Get the compass indication for a certain direction.
	 * i.e. 90 becomes 'E'
	 *
	 * @param string
	 * @return string
	 * @private
	 */
	function getCompassDirection ($input)
	{
		if ($input == 'VRB')
		{
			return $input;
		}
		$compass = array ('N', 'NNE', 'NE', 'ENE', 'E', 'ESE', 
					'SE', 'SSE', 'S', 'SSW', 'SW', 'WSW', 'W', 'WNW', 'NW', 'NNW');
		return $compass [round ($input / 22.5) % 16];
	}
	/**
	 * Get (horizontal ground) visibility information in meters, 
	 * indicating the worst value around the observation point
	 * 0000 means less than 50 meter, 1700 means 1700 meters, 9999
	 * indicates more than 10 km.
	 * 
	 * If the visibility is in miles AND fractional, the fraction is 
	 * seperated by a space. This screws up our initial split on spaces
	 * so we do a check on whether we have received a single integer 
	 * (in the calling function) and if this is the case, the function 
	 * is called a second time to determine the fraction)
	 * 
	 * @param string input
	 * @return array
	 * @private
	 */
	function getVisibility ($input)
	{
		$result = null;
		if (substr ($input, -2) == 'SM')
		{
			//
			// Visibility in (statute) miles
			//
			$result = array ();
			$result ['units']='miles';
			$result ['visibility'] = (substr ($input, 0, strlen ($input)-2));
		}
		else if (strlen ($input) == 4)
		{
			//
			// Visibility in meters
			//
			$result = array ();
			$result ['units'] = 'meters';
			$result ['visibility'] = $input;
		}
		else 
		{
			// 
			// Normally, this should be the fraction specific case?
			// TO TEST
			// Anyway, we can test later on if no units are set and the
			// visibility strlen is 1
			//
			$result = array ();
			$result ['visibility'] = $input;
		}
		return $result;
	}
	
	/**
	 * Format: 	Rdd/vvvvFT  or Rdd/vvvvVvvvvFT
	 * Attention, multiple runways may be included in a Metar string!!!
	 *
	 * @param string $input
	 * @return unknown
	 * @private
	 */
	function getRunwayInfo ($input)
	{
		$result = null;
		if (substr ($input, 0, 1) == 'R')
		{
			$result = array ();
			$split = explode ('/', $input);
			//
			// Actually the direction in 10th of degrees where 09 is east,
			// 27 west etc. If there are multiple runways in the same 
			// direction, a character is appended 
			// (R for Right, C for Center, L for Left)
			//
			$result ['runwayDesignator'] = substr ($split[0], 1, 2);			
			if (strstr ($split[1], 'FT'))
			{
				$visibility = $substr ($split[1], 0, strlen ($split[1], -2));
				$result ['units'] = 'feet';
			}
			else
			{
				$visibility = $split[1];
				$result ['units'] = 'meters';
			}
			if (strstr ($visibility, 'V'))
			{
				$visibilityArray = explode ('V', $visibility);
				$result ['visibilityFrom'] = 
					$this->getVisibilityTendency ($visibilityArray [0]);
				$result ['visibilityTo'] = 
					$this->getVisibilityTendency ($visibilityArray [1]);
			}
			else
			{
				$result ['visibility'] = 
					$this->getVisibilityTendency ($visibility);
			}
		}
		return $result;
	}

	/**
	 * Function used from within getRunwayVisibility. It parses the visibility
	 * and retrieves the tendency (if available) from it.
	 * 
	 * The result array is an array with at least the visibility parameter 
	 * and optionally the tendency parameter
	 *
	 * @param string input
	 * @return array
	 * @private
	 */
	function getVisibilityTendency ($input)
	{
		$result = array ();
		$tendency = substr ($input, strlen ($input) -1);
		if ($tendency == 'D')
		{
			$result ['tendency'] = 'decreasing';
			$result ['visibility'] = substr ($input, 0, strlen ($input) -1);
		}
		else if ($tendency == 'U')
		{
			$result ['tendency'] = 'increasing';
			$result ['visibility'] = substr ($input, 0, strlen ($input) -1);
		}
		else if ($tendency == 'N')
		{
			$result ['tendency'] = 'no change';
			$result ['visibility'] = substr ($input, 0, strlen ($input) -1);
		}
		else
		{
			$result ['visibility'] = $input;
		}
		return $result;
	}


	/**
	 * Splits the weather conditions. The split is deliberately not 
	 * translated, an i18n process at another level should do this.
	 *
	 * @param string
	 * @return array
	 * @private
	 */
	function getWeatherConditions ($input)
	{
		$result = null;
		if (ereg ('^(-|\+|VC)?(TS|SH|FZ|BL|DR|MI|BC|PR|RA|DZ|SN|SG|GR|GS|PE|IC|UP|BR|FG|FU|VA|DU|SA|HZ|PY|PO|SQ|FC|SS|DS)+$',$input))
		{
			$result = array ();
			//
			// First get the intensity
			//
			if (ereg ('^(-|\+|VC)', $input, $split))
			{
				$intensity = substr ($input, 0, strlen ($split[0]));	
				switch ($intensity)
				{
					case '-':
						$result ['intensity'] = 'light';
						break;
					case '+':
						$result ['intensity'] = 'heavy';
						break;
					case 'VC':
						$result ['intensity'] = 'vicinity';
						break;
				}
				$result ['intensityElement'] = $intensity;
				$input = substr ($input, strlen ($split[0]));
			}
			else
			{
				$result ['intensity']='moderate';
			}
			//
			// Now get the description
			//
			if (ereg ('(MI|PR|BC|DR|BL|SH|TS|FZ)', $input, $split))
			{
				$result ['descriptionElement'] = $split[0];
				switch ($split[0])
				{
					case 'MI':
						$result ['description'] = 'shallow';
						break;
					case 'PR':
						$result ['description'] = 'partial';
						break;
					case 'BC':
						$result ['description'] = 'patches';
						break;
					case 'DR':
						$result ['description'] = 'low_drifting';
						break;
					case 'BL':
						$result ['description'] = 'blowing';
						break;
					case 'SH':
						$result ['description'] = 'showers';
						break;
					case 'TS':
						$result ['description'] = 'thunderstorm';
						break;
					case 'FZ':
						$result ['description'] = 'freezing';
						break;
				}
				$input = substr ($input, 2);
			}	
			//
			// Now get the precipation
			//
			if (ereg ('(DZ|RA|SN|SG|IC|PL|GR|GS|UP)', $input, $split))
			{
				$result ['precipationElement'] = $split[0];
				switch ($split[0])
				{
					case 'DZ':
						$result ['precipation'] = 'drizzel';
						break;
					case 'RA':
						$result ['precipation'] = 'rain';
						break;
					case 'SN':
						$result ['precipation'] = 'snow';
						break;
					case 'SG':
						$result ['precipation'] = 'snow_grains';
						break;
					case 'IC':
						$result ['precipation'] = 'ice_pellets';
						break;
					case 'GR':
						$result ['precipation'] = 'hail';
						break;
					case 'GS':
						$result ['precipation'] = 'small_hail';
						break;
					case 'UP':
						$result ['precipation'] = 'unknown';
						break;
				}
				$input = substr ($input, 2);
			}
			//
			// Get the Obscuration
			//
			if (ereg ('(BR|FG|FU|VA|DU|SA|HZ|PY)', $input, $split))
			{
				$result ['obscurationElement'] = $split[0];
				switch ($split[0])
				{
					case 'BR':
						$result ['obscuration'] = 'mist';
						break;
					case 'FG':
						$result ['obscuration'] = 'fog';
						break;
					case 'FU':
						$result ['obscuration'] = 'smoke';
						break;
					case 'VA':
						$result ['obscuration'] = 'volcanic_ash';
						break;
					case 'DU':
						$result ['obscuration'] = 'widespread_dust';
						break;
					case 'SA':
						$result ['obscuration'] = 'sand';
						break;
					case 'HZ':
						$result ['obscuration'] = 'haze';
						break;
					case 'PY':
						$result ['obscuration'] = 'spray';
						break;
				}
				$input = substr ($input, 2);
			}
			//
			// Get the other info
			//
			if (ereg ('(PO|SQ|FC|SS|DS)', $input, $split))
			{
				$result ['otherElement'] = $split[0];
				switch ($split[0])
				{
					case 'PO':
						$result ['other'] = 'well_developed_dust_sand_whirls';
						break;
					case 'SQ':
						$result ['other'] = 'squalls';
						break;
					case 'FC':
						$result ['other'] = 'funnel_clouds_inc_tornadoes_or_waterspouts';
						break;
					case 'SS':
						$result ['other'] = 'sandstorm';
						break;
					case 'DS':
						$result ['other'] = 'duststorm';
						break;
				}
				$input = substr ($input, 2);
			}
			//
			// input length should be zero at this point
			//
		}
		return $result;
	}


	/**
 	 * Gets the cloud coverage information.
	 * Format: Format: nnnhhh or nnhhh if nn == VV or SKC or CLR or NSC or NCD or CAVOK
	 * 
 	 * CAVOK indicated 'CAV ok' (CAV = ceiling and visibility')
	 * Where nnn indicates the sky condition
	 * 
	 * There might be multiple cloudcoverage sections in a metar string!
	 *
	 * SKC 	(0/8)
	 * CLR 	(0/8)
	 * FEW 	(1/8, 2/8)
	 * SCT 	(3/8, 4/8)
	 * BKN 	(5/8, 6/8, 7/8)
	 * OVC 	(8/8)
	 * VV 	(8/8) - vertical visibility.
	 *
	 * And hhh indicates the cloud height in flight level (100 feet units) 
	 *
 	 * Also seen:
	 * NSC (no significant cloud coverage).	
	 * NCD (no cloud detected).	
	 */
	function getCloudCoverage ($input)
	{
		$result = null;
		//
		// Clear sky
		//
		if ($input == 'SKC' || $input == 'CLR')
		{
			$result = array ();
			$result ['coverageElement'] = $input;
			$result ['coverageMeaning'] = 'clear';
		}
		else if ($input == 'CAVOK')
		{
			$result = array ();
			$result ['coverageElement'] = $input;
			$result ['coverageMeaning'] = 'ceiling_and_visibility_ok';
		}
		else if ($input == 'NSC')
		{
			$result = array ();
			$result ['coverageElement'] = $input;
			$result ['coverageMeaning'] = 'no_significant_cloud_coverage';
		}
		else if ($input == 'NCD')
		{
			$result = array ();
			$result ['coverageElement'] = $input;
			$result ['coverageMeaning'] = 'no_cloud_detected';
		}
		//
		// Vertical visibility
		//
		else if (strstr ($input, 'VV'))
		{
			$result = array ();
			$result ['coverageElement'] = 'VV';
			$result ['coverageMeaning'] = 'vertical_visibility';
			$coverage = substr ($input, 2);
			if (strstr ($coverage, 'CB'))
			{
				$coverage = substr ($coverage, 0, strlen ($coverage-2));
				$result ['cumul'] = 'cumulonimbus';
			}
			else if (strstr ($coverage, 'TCU'))
			{
				$coverage = substr ($coverage, 0, strlen ($coverage-3));
				$result ['cumul'] = 'towering_cumulus';
				$result ['coverageIndex'] = '3';
			}
			$result ['coverage'] = $coverage;
		}	
		//
		// Other cases
		//
		else if (strstr ($input, 'FEW') || strstr ($input, 'SCT') ||
				 strstr ($input, 'BKN') || strstr ($input, 'OVC'))
		{
			$result = array ();
			$elem = substr ($input, 0, 3);
			$coverage = substr ($input, 3);
			$result ['coverageElement'] = $elem;
			switch ($elem)
			{
				case 'FEW':
					$result ['coverageMeaning'] = 'few';
					$result ['coverageIndex'] = '1';
					break;
				case 'SCT':
					$result ['coverageMeaning'] = 'scattered';
					$result ['coverageIndex'] = '2';
					break;
				case 'BKN':
					$result ['coverageMeaning'] = 'broken';
					$result ['coverageIndex'] = '3';
					break;
				case 'OVC':
					$result ['coverageMeaning'] = 'overcast';
					$result ['coverageIndex'] = '4';
					break;
			}
			if (strstr ($coverage, 'CB'))
			{
				$coverage = substr ($coverage, 0, strlen ($coverage-2));
				$result ['cumul'] = 'cumulonimbus';
			}
			else if (strstr ($coverage, 'TCU'))
			{
				$coverage = substr ($coverage, 0, strlen ($coverage-3));
				$result ['cumul'] = 'towering_cumulus';
			}
			$result ['coverage'] = $coverage;
		}
		return $result;
	}

	/**
	 * Gets the temperature and dewpoint information
	 *
	 * Format: Format: tt/dd
	 * Where tt is the temperature in celsius and 
	 * dd is the dewpoint in degrees celsius
	 * Both temperature and dewpoint can be prepenced by an
	 * 'M' indicating a negative temperature
 	 *	
	 * @param string
	 * @return array
	 * @private
	 */
	function getTemperature ($input)
	{
		$result = null;
		if (strstr ($input, '/'))
		{
			$result = array ();
			$split = explode ('/', $input);
			$temp = $split [0];
			$dew = $split [1];
			if (strstr ($temp, 'M'))
			{
				$result ['temperature'] = '-'.substr ($temp, 1);
			}
			else
			{
				$result ['temperature'] = $temp;
			}
			if (strstr ($dew, 'M'))
			{
				$result ['dewpoint'] = '-'.substr ($dew, 1);
			}
			else
			{
				$result ['dewpoint'] = $dew;
			}
		}
		return $result;
	}

	/**
	 * Get the atmospheric pressure
	 * Format: Qpppp or Ahhhh
	 * Where Q indicates presure measurement in hPa (or millibars)
	 * and A indicates presure measurement in Hg, multiplied by 100
	 *
	 * @param string
	 * @return array
	 * @private
	 */
	function getPressure ($input)
	{
		$result = null;
		if (strstr ($input, 'Q') || strstr ($input, 'A'))
		{
			$result = array ();
			$pressure = substr ($input, 1);
			if (strstr ($input, 'A'))
			{
				// inches in mercury
				$result ['units'] = 'inHg';
				$result ['pressure'] = $pressure / 100;
			}
			else
			{
				$result ['units'] = 'hPa';
				$result ['pressure'] = $pressure;
			}
		}
		return $result;
	}
	
	/**
	 * Returns the validity of the forecast.
	 * The validity is in the form of DDHHHH
	 *
	 * @param unknown_type $input
	 * @return unknown
	 */
	function getValidity ($input)
	{
		$result = array ();
		$result ['day'] = substr ($input, 0, 2);
		$result ['fromHours'] = substr ($input, 2, 2);
		$result ['untilHours'] = substr ($input, 4, 2);
		return $result;
	}
	
	/**
	 * The modifier (AMD, COR, AMD COR or RTD). Returns the modifier if found,
	 * null otherwise
	 * 
	 * AMD means Forecast was amended
	 * COR means Forecast was corrected
	 * AMD COR means that the Forecast was first ammended then corrected
	 * RTD means that the Forecast was transmitted late
	 *
	 * @param string $input
	 * @return string
	 */
	function getModifier ($input)
	{
		$result = null;
		if (strstr ($input, 'AMD') || strstr ($input, 'COR') || strstr ($input, 'RTD'))
		{
			$result = $input;
		}
		return $result;
	}
	
	// Height in hunders of feet?
	// Thickness in 1000s of feet?
	function getIcingGroup ($input)
	{
		if (!substr ($input, 0, 1) == 6)
		{
			return null;
		}
		$result = array ();
		$result ['indicator'] = substr ($input, 1, 1);
		$result ['height'] = substr ($input, 2, 3);
		$result ['thickness'] = substr ($input, 5, 1);
		return $result;
	}
	
	// Height in hunders of feet?
	// Thickness in 1000s of feet?
	function getTurbulence ($input)
	{
		if (!substr ($input, 0, 1) == 5)
		{
			return null;
		}
		$result = array ();
		$result ['indicator'] = substr ($input, 1, 1);
		$result ['height'] = substr ($input, 2, 3);
		$result ['thickness'] = substr ($input, 5, 1);
		return $result;
	}
	
	/**
	 * Wind shear is the forecast of non-convective low level 
	 * winds (up to 2000 feet) and is entered after the sky 
	 * conditions when wind shear is epxected. The forecast 
	 * includes the height of the wind shear followed by the 
	 * wind direction and wind speed at the indicated height. 
	 * 
	 * Height is given in hundreds of feet AGL up to and 
	 * including 2,000 feet. Wind shear is encoded with the 
	 * contraction WS followed by a three-digit height, 
	 * slant character, and winds at the height indicated in 
	 * the same format as surface winds. The wind shear 
	 * element is omitted if not expected to occur.
	 *
	 * @param string $input
	 * @return array
	 */
	function getWindShear ($input)
	{
		$result = null;
		if (substr ($input, 0, 2) == 'WS')
		{
			$result = array ();
			$split = explode ('/', substr ($input, 2));
			$result ['height'] = $split [0];
			$result ['wind'] = $this->getWind ($split[1]);
		}
		return null;
	}
	
	function getProbability ($input)
	{
		if (strstr ($input, 'PROB'))
		{
			return substr ($input, 4);
		}
		return -1;
	}
}
?>
