<?php
/**
 * (Yet another) IP subnet calculator
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - July 2004
 * @package org.brim-project.plugins.webtools
 * @subpackage i18n
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2006 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class SubnetCalculator
{
	/**
	 * Empty constructor
	 */
	function SubnetCalculator ()
	{
	}

	/**
	 * Calculates subnet information for the given ip address
	 *
	 * @param string ipAddress the ipaddress with subnet info
	 * 		This address is expected to be in the following
	 *		format: 10.0.0.1/16
	 * @return array the information with the following layout:
	 *		[errors] => array
	 *		[input] => string
	 *		[ipAddress]  => string
	 * 		[netmask]  => string
	 * 		[numberOfHosts] => integer
	 * 		[networkAddress] => string
	 *		[broadcastAddress] => string
	 */
	function calculate ($subnetInfo)
	{
		$result = array ();
		$errors = array ();
		$result['input']=$subnetInfo;

		if (!$this->validateCIDR(trim($subnetInfo)))
		{
			$errors['invalidCIDR']=$subnetInfo;
		}
		if (!$this->validateIp (trim ($subnetInfo)))
		{
			$errors['invalidIP']=$subnetInfo;
		}
		$result['errors']=$errors;

		//
		// No need to continue if we have errors
		//
		if (count($errors) > 0)
		{
			return $result;
		}
		$host = strtok ($subnetInfo,'/');
		$cidr = strtok ('/');

		$result['ipAddress']=$host;
		$result['netmask']= $this->getNetmaskForCIDR($cidr);
		$result['numberOfHosts']=bindec(str_pad('',32-$cidr,1))-1;
		$result['networkAddress']=
			$this->getNetworkForIpAndCIDR ($host, $cidr);
		$result['broadcastAddress']=
			$this->getBroadcastForIpAndCIDR ($host, $cidr);
		return $result;
	}

	/**
	 * Checks whether the input is syntactically correct by checking
	 * for an appropriate CIDR number (Classless Inter Domain Routing).
	 * This is the number after the ip address
	 * (i.e. 10.0.0.1/8 => CIDR = '/8') which must be beteen 1 and 32
	 *
	 * @param string subnetInfo the subnet input information
	 * @return boolean <code>true</code> if the input is
	 *		syntactically correct, <code>false</code> otherwise
	 */
	function validateCIDR ($subnetInfo)
	{
		if (ereg ('/', $subnetInfo))
		{
			$host = strtok ($subnetInfo, '/');
			$mask = strtok ('/');
			return ($mask >= 1 && $mask <= 32);
		}
		return false;
	}

	/**
	 * Validates an ip address (IPv4) by chopping it up based
	 * on the dots in the address. Each of the four resulting parts
	 * must be a number between 0 and 255
	 *
	 * @param string subnetInfo the information (including CIDR)
	 * @return boolean <code>true</code> if it is a valid ip
	 * 		address, <code>false</code> otherwise
	 */
	function validateIp ($subnetInfo)
	{
		$host = strtok ($subnetInfo, '/');
		$a = strtok ($host, '.');
		$b = strtok ('.');
		$c = strtok ('.');
		$d = strtok ('.');

		return (is_numeric ($a) && $a >= 0 && $a <= 255 &&
	   			is_numeric ($b) && $b >= 0 && $b <= 255 &&
				is_numeric ($c) && $c >= 0 && $c <= 255 &&
				is_numeric ($d) && $d >= 0 && $d <= 255);
	}


	/**
	 * Returns the netmask in form of an ip-address based on the
	 * CIDR input
	 *
	 * @param integer cidr the cidr netmask notation
	 * @return string the netmask in dotted notation
	 */
	function getNetmaskForCIDR ($cidr)
	{
		$binaryForm = str_pad (str_pad ('', $cidr, '1'), 32, '0');
		return $this->binaryToIp ($binaryForm);
	}


	/**
	 * Returns the network addres in form of an ip-address based on the
	 * CIDR input in combination with the hosts ip address
	 *
	 * @param string host the ip address (dotted notation)
	 * @param integer cidr the cidr netmask notation
	 * @return string the netmask in dotted notation
	 */
	function getNetworkForIpAndCIDR ($host, $cidr)
	{
		$binaryHost = $this->getIpInBinary($host);
		$binaryForm = str_pad (substr ($binaryHost, 0, $cidr), 32, 0);
		return $this->binaryToIp ($binaryForm);
	}


	/**
	 * Retrieve the broadcast address for the ip address
	 * with cidr
	 *
	 * @param string host the ip address
	 * @param integer cidr the cidr notation (a number between 0 and 32)
	 * @return the broadcast address for this ip address
	 */
	function getBroadcastForIpAndCIDR ($host, $cidr)
	{
		$binaryHost = $this->getIpInBinary ($host);
		$binaryForm = str_pad (substr ($binaryHost, 0, $cidr), 32, 1);
		return $this->binaryToIp ($binaryForm);
	}

	/**
	 * Returns an ip address in binary form.
	 * As example: 192.168.0.1 becomes
	 * 11000000101010000000000000000001
	 * The routine splits the ip address based on its dots,
	 * retrieves the binary form of the specific number and left
	 * pads it with zeros
	 *
	 * @param string ipaAddress in the form a.b.c.d where a, b, c and
	 * 		d are numbers between 0 and 255
	 * @return string the binary form of the ip address, a string with
	 *		length 32 containing only '0's and '1's.
	 */
	function getIpInBinary ($ipAddress)
	{
		$temp = array ();
		$splitted = explode ('.',$ipAddress);
		for ($i=0; $i<4; $i++)
		{
			$temp [$i] = str_pad (decbin ($splitted[$i]), 8, '0',
				STR_PAD_LEFT);
		}
		return implode ('', $temp);
	}

	/**
	 * Returns a binary ip address into a dotted notation.
	 * as example:
	 * 11000000101010000000000000000001
	 * becomes 192.168.0.1
	 *
	 * @param string binaryIp a string of 32 characters containing only
	 * 		'0's and '1's. This string represents the binary format of
	 *		an ip-address
	 * @return string the dotted notation of the ip address
	 */
	function binaryToIp ($binaryIp)
	{
		$temp = array ();
		$splitted = explode ('*', chunk_split ($binaryIp, 8, '*'));
		for ($i=0; $i<4; $i++)
		{
			$temp[$i] = bindec ($splitted[$i]);
		}
		return implode ('.', $temp);
	}
}
?>